@@ -28,6 +28,7 @@ import torch
 import pytorch_lightning as pl
 from pytorch_lightning.callbacks.model_checkpoint import ModelCheckpoint
 from pytorch_lightning.utilities import rank_zero_only
+from pytorch_lightning.utilities.warnings import rank_zero_deprecation
 
 
 def rank_zero_experiment(fn: Callable) -> Callable:
@@ -45,8 +46,7 @@ def rank_zero_experiment(fn: Callable) -> Callable:
 
 
 class LightningLoggerBase(ABC):
-    """
-    Base class for experiment loggers.
+    """Base class for experiment loggers.
 
     Args:
         agg_key_funcs:
@@ -73,8 +73,7 @@ class LightningLoggerBase(ABC):
         self._agg_default_func = agg_default_func
 
     def after_save_checkpoint(self, checkpoint_callback: "ReferenceType[ModelCheckpoint]") -> None:
-        """
-        Called after model checkpoint callback saves a new checkpoint
+        """Called after model checkpoint callback saves a new checkpoint.
 
         Args:
             checkpoint_callback: the model checkpoint callback instance
@@ -86,8 +85,7 @@ class LightningLoggerBase(ABC):
         agg_key_funcs: Optional[Mapping[str, Callable[[Sequence[float]], float]]] = None,
         agg_default_func: Callable[[Sequence[float]], float] = np.mean,
     ):
-        """
-        Update aggregation methods.
+        """Update aggregation methods.
 
         Args:
             agg_key_funcs:
@@ -111,8 +109,7 @@ class LightningLoggerBase(ABC):
     def _aggregate_metrics(
         self, metrics: Dict[str, float], step: Optional[int] = None
     ) -> Tuple[int, Optional[Dict[str, float]]]:
-        """
-        Aggregates metrics.
+        """Aggregates metrics.
 
         Args:
             metrics: Dictionary with metric names as keys and measured quantities as values
@@ -155,9 +152,7 @@ class LightningLoggerBase(ABC):
             self.log_metrics(metrics=metrics_to_log, step=agg_step)
 
     def agg_and_log_metrics(self, metrics: Dict[str, float], step: Optional[int] = None):
-        """
-        Aggregates and records metrics.
-        This method doesn't log the passed metrics instantaneously, but instead
+        """Aggregates and records metrics. This method doesn't log the passed metrics instantaneously, but instead
         it aggregates them and logs only if metrics are ready to be logged.
 
         Args:
@@ -196,8 +191,7 @@ class LightningLoggerBase(ABC):
 
     @staticmethod
     def _sanitize_callable_params(params: Dict[str, Any]) -> Dict[str, Any]:
-        """
-        Sanitize callable params dict, e.g. ``{'a': <function_**** at 0x****>} -> {'a': 'function_****'}``.
+        """Sanitize callable params dict, e.g. ``{'a': <function_**** at 0x****>} -> {'a': 'function_****'}``.
 
         Args:
             params: Dictionary containing the hyperparameters
@@ -223,8 +217,7 @@ class LightningLoggerBase(ABC):
 
     @staticmethod
     def _flatten_dict(params: Dict[Any, Any], delimiter: str = "/") -> Dict[str, Any]:
-        """
-        Flatten hierarchical dict, e.g. ``{'a': {'b': 'c'}} -> {'a/b': 'c'}``.
+        """Flatten hierarchical dict, e.g. ``{'a': {'b': 'c'}} -> {'a/b': 'c'}``.
 
         Args:
             params: Dictionary containing the hyperparameters
@@ -259,8 +252,7 @@ class LightningLoggerBase(ABC):
 
     @staticmethod
     def _sanitize_params(params: Dict[str, Any]) -> Dict[str, Any]:
-        """
-        Returns params with non-primitvies converted to strings for logging.
+        """Returns params with non-primitvies converted to strings for logging.
 
         >>> params = {"float": 0.3,
         ...           "int": 1,
@@ -289,8 +281,7 @@ class LightningLoggerBase(ABC):
 
     @abstractmethod
     def log_hyperparams(self, params: argparse.Namespace, *args, **kwargs):
-        """
-        Record hyperparameters.
+        """Record hyperparameters.
 
         Args:
             params: :class:`~argparse.Namespace` containing the hyperparameters
@@ -299,8 +290,7 @@ class LightningLoggerBase(ABC):
         """
 
     def log_graph(self, model: "pl.LightningModule", input_array=None) -> None:
-        """
-        Record model graph
+        """Record model graph.
 
         Args:
             model: lightning model
@@ -308,13 +298,26 @@ class LightningLoggerBase(ABC):
         """
         pass
 
+    def log_text(self, *args, **kwargs) -> None:
+        """Log text.
+
+        Arguments are directly passed to the logger.
+        """
+        raise NotImplementedError
+
+    def log_image(self, *args, **kwargs) -> None:
+        """Log image.
+
+        Arguments are directly passed to the logger.
+        """
+        raise NotImplementedError
+
     def save(self) -> None:
         """Save log data."""
         self._finalize_agg_metrics()
 
     def finalize(self, status: str) -> None:
-        """
-        Do any processing that is necessary to finalize an experiment.
+        """Do any processing that is necessary to finalize an experiment.
 
         Args:
             status: Status that the experiment finished with (e.g. success, failed, aborted)
@@ -322,17 +325,31 @@ class LightningLoggerBase(ABC):
         self.save()
 
     def close(self) -> None:
-        """Do any cleanup that is necessary to close an experiment."""
+        """Do any cleanup that is necessary to close an experiment.
+
+        See deprecation warning below.
+
+        .. deprecated:: v1.5
+            This method is deprecated in v1.5 and will be removed in v1.7.
+            Please use `LightningLoggerBase.finalize` instead.
+        """
+        rank_zero_deprecation(
+            "`LightningLoggerBase.close` method is deprecated in v1.5 and will be removed in v1.7."
+            " Please use `LightningLoggerBase.finalize` instead."
+        )
         self.save()
 
     @property
     def save_dir(self) -> Optional[str]:
-        """
-        Return the root directory where experiment logs get saved, or `None` if the logger does not
-        save data locally.
-        """
+        """Return the root directory where experiment logs get saved, or `None` if the logger does not save data
+        locally."""
         return None
 
+    @property
+    def group_separator(self):
+        """Return the default separator used by the logger to group the data into subfolders."""
+        return "/"
+
     @property
     @abstractmethod
     def name(self) -> str:
@@ -351,9 +368,7 @@ class LightningLoggerBase(ABC):
 
 
 class LoggerCollection(LightningLoggerBase):
-    """
-    The :class:`LoggerCollection` class is used to iterate all logging actions over
-    the given `logger_iterable`.
+    """The :class:`LoggerCollection` class is used to iterate all logging actions over the given `logger_iterable`.
 
     Args:
         logger_iterable: An iterable collection of loggers
@@ -380,9 +395,7 @@ class LoggerCollection(LightningLoggerBase):
 
     @property
     def experiment(self) -> List[Any]:
-        """
-        Returns a list of experiment objects for all the loggers in the logger collection.
-        """
+        """Returns a list of experiment objects for all the loggers in the logger collection."""
         return [logger.experiment for logger in self._logger_iterable]
 
     def agg_and_log_metrics(self, metrics: Dict[str, float], step: Optional[int] = None):
@@ -401,6 +414,14 @@ class LoggerCollection(LightningLoggerBase):
         for logger in self._logger_iterable:
             logger.log_graph(model, input_array)
 
+    def log_text(self, *args, **kwargs) -> None:
+        for logger in self._logger_iterable:
+            logger.log_text(*args, **kwargs)
+
+    def log_image(self, *args, **kwargs) -> None:
+        for logger in self._logger_iterable:
+            logger.log_image(*args, **kwargs)
+
     def save(self) -> None:
         for logger in self._logger_iterable:
             logger.save()
@@ -410,34 +431,38 @@ class LoggerCollection(LightningLoggerBase):
             logger.finalize(status)
 
     def close(self) -> None:
+        """
+        .. deprecated:: v1.5
+            This method is deprecated in v1.5 and will be removed in v1.7.
+            Please use `LoggerCollection.finalize` instead.
+        """
+        rank_zero_deprecation(
+            "`LoggerCollection.close` method is deprecated in v1.5 and will be removed in v1.7."
+            " Please use `LoggerCollection.finalize` instead."
+        )
         for logger in self._logger_iterable:
             logger.close()
 
     @property
     def save_dir(self) -> Optional[str]:
-        """
-        Returns ``None`` as checkpoints should be saved to default / chosen location when using multiple loggers.
-        """
+        """Returns ``None`` as checkpoints should be saved to default / chosen location when using multiple
+        loggers."""
         # Checkpoints should be saved to default / chosen location when using multiple loggers
         return None
 
     @property
     def name(self) -> str:
-        """
-        Returns the experiment names for all the loggers in the logger collection joined by an underscore.
-        """
+        """Returns the experiment names for all the loggers in the logger collection joined by an underscore."""
         return "_".join(str(logger.name) for logger in self._logger_iterable)
 
     @property
     def version(self) -> str:
-        """
-        Returns the experiment versions for all the loggers in the logger collection joined by an underscore.
-        """
+        """Returns the experiment versions for all the loggers in the logger collection joined by an underscore."""
         return "_".join(str(logger.version) for logger in self._logger_iterable)
 
 
 class DummyExperiment:
-    """Dummy experiment"""
+    """Dummy experiment."""
 
     def nop(self, *args, **kw):
         pass
@@ -451,9 +476,9 @@ class DummyExperiment:
 
 
 class DummyLogger(LightningLoggerBase):
-    """
-    Dummy logger for internal use. It is useful if we want to disable user's
-    logger for a feature, but still ensure that user code can run
+    """Dummy logger for internal use.
+
+    It is useful if we want to disable user's logger for a feature, but still ensure that user code can run
     """
 
     def __init__(self):
@@ -485,15 +510,18 @@ class DummyLogger(LightningLoggerBase):
         # enables self.logger[0].experiment.add_image(...)
         return self
 
+    def __iter__(self):
+        # if DummyLogger is substituting a logger collection, pretend it is empty
+        yield from ()
+
 
 def merge_dicts(
     dicts: Sequence[Mapping],
     agg_key_funcs: Optional[Mapping[str, Callable[[Sequence[float]], float]]] = None,
     default_func: Callable[[Sequence[float]], float] = np.mean,
 ) -> Dict:
-    """
-    Merge a sequence with dictionaries into one dictionary by aggregating the
-    same keys with some given function.
+    """Merge a sequence with dictionaries into one dictionary by aggregating the same keys with some given
+    function.
 
     Args:
         dicts:
