@@ -11,49 +11,60 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 import logging
-from contextlib import suppress
 from typing import Optional
 
 from pytorch_lightning.loops import Loop
 from pytorch_lightning.loops.epoch import TrainingEpochLoop
+from pytorch_lightning.loops.utilities import _is_max_limit_reached
 from pytorch_lightning.trainer.connectors.logger_connector.result import ResultCollection
 from pytorch_lightning.trainer.progress import Progress
 from pytorch_lightning.trainer.supporters import TensorRunningAccum
+from pytorch_lightning.utilities import rank_zero_deprecation
+from pytorch_lightning.utilities.exceptions import MisconfigurationException
 
 log = logging.getLogger(__name__)
 
 
 class FitLoop(Loop):
-    """
-    This Loop iterates over the epochs to run the training.
+    """This Loop iterates over the epochs to run the training.
 
     Args:
         min_epochs: The minimum number of epochs
-        max_epochs: The maximum number of epochs
+        max_epochs: The maximum number of epochs, can be set -1 to turn this limit off
     """
 
-    def __init__(self, min_epochs: Optional[int] = None, max_epochs: Optional[int] = None):
+    def __init__(
+        self,
+        min_epochs: Optional[int] = 1,
+        max_epochs: int = 1000,
+    ) -> None:
         super().__init__()
+        if max_epochs < -1:
+            # Allow max_epochs to be zero, since this will be handled by fit_loop.done
+            raise MisconfigurationException(
+                f"`max_epochs` must be a non-negative integer or -1. You passed in {max_epochs}."
+            )
+
         self.max_epochs = max_epochs
         self.min_epochs = min_epochs
         self.epoch_loop: Optional[TrainingEpochLoop] = None
         self.epoch_progress = Progress()
+        self._is_fresh_start_epoch: bool = True
 
     @property
     def current_epoch(self) -> int:
-        """Return the current epoch"""
+        """Return the current epoch."""
         return self.epoch_progress.current.completed
 
     @current_epoch.setter
     def current_epoch(self, value: int) -> None:
-        """Setter for the current epoch"""
+        """Setter for the current epoch."""
         self.epoch_progress.current.completed = value
 
     @property
     def global_step(self) -> int:
-        """Returns the global step"""
+        """Returns the global step."""
         return self.epoch_loop.global_step
 
     @global_step.setter
@@ -63,23 +74,23 @@ class FitLoop(Loop):
 
     @property
     def total_batch_idx(self) -> int:
-        """Returns the total number of batches already run (across all epochs)"""
+        """Returns the current batch index (across epochs)"""
         return self.epoch_loop.total_batch_idx
 
     @property
     def batch_idx(self) -> int:
-        """Returns the number of batches already run within this epoch"""
-        return self.epoch_loop.batch_progress.current.ready - 1
+        """Returns the current batch index (within this epoch)"""
+        return self.epoch_loop.batch_idx
 
     @property
     def split_idx(self) -> int:
-        """Returns the index of the current batch split (within the current batch) for bptt"""
+        """Returns the index of the current batch split (within the current batch) for bptt."""
         return self.epoch_loop.batch_loop.split_idx
 
     @property
     def min_steps(self) -> int:
         # TODO(@justusschock): Why aren't we using the attribute in this class?
-        """Returns the minimum numnber of steps to run"""
+        """Returns the minimum numnber of steps to run."""
         return self.epoch_loop.min_steps
 
     @min_steps.setter
@@ -90,29 +101,43 @@ class FitLoop(Loop):
 
     @property
     def max_steps(self) -> int:
-        """Returns the maximum number of steps to run"""
+        """Returns the maximum number of steps to run."""
         return self.epoch_loop.max_steps
 
     @max_steps.setter
     def max_steps(self, value: int) -> None:
         """Sets the maximum number of steps (forwards to epoch_loop)"""
         # TODO(@awaelchli): This setter is required by debugging connector (fast dev run), should be avoided
+        if value is None:
+            rank_zero_deprecation(
+                "Setting `max_steps = None` is deprecated in v1.5 and will no longer be supported in v1.7."
+                " Use `max_steps = -1` instead."
+            )
+            value = -1
+        elif value < -1:
+            raise MisconfigurationException(
+                f"`max_steps` must be a non-negative integer or -1 (infinite steps). You passed in {value}."
+            )
         self.epoch_loop.max_steps = value
 
     @property
     def running_loss(self) -> TensorRunningAccum:
-        """Returns the running loss"""
+        """Returns the running loss."""
         return self.epoch_loop.batch_loop.running_loss
 
     @property
     def _skip_backward(self) -> bool:
         """Determines whether the loop will skip backward during automatic optimization."""
-        return self.epoch_loop.batch_loop._skip_backward
+        assert self.epoch_loop.batch_loop is not None
+        assert self.epoch_loop.batch_loop.optimizer_loop is not None
+        return self.epoch_loop.batch_loop.optimizer_loop._skip_backward
 
     @_skip_backward.setter
     def _skip_backward(self, value: bool) -> None:
         """Determines whether the loop will skip backward during automatic optimization."""
-        self.epoch_loop.batch_loop._skip_backward = value
+        assert self.epoch_loop.batch_loop is not None
+        assert self.epoch_loop.batch_loop.optimizer_loop is not None
+        self.epoch_loop.batch_loop.optimizer_loop._skip_backward = value
 
     @property
     def _results(self) -> ResultCollection:
@@ -126,12 +151,12 @@ class FitLoop(Loop):
     def done(self) -> bool:
         """Evaluates when to leave the loop.
 
-        Returns True if trainer.should_stop was set (e.g. by early stopping)
-        or if the maximum number of steps or epochs is reached.
+        Returns True if trainer.should_stop was set (e.g. by early stopping) or if the maximum number of steps or epochs
+        is reached.
         """
         # TODO(@awaelchli): Move track steps inside training loop and move part of these condition inside training loop
-        stop_steps = self.max_steps is not None and self.global_step >= self.max_steps
-        stop_epochs = self.max_epochs is not None and self.current_epoch >= self.max_epochs
+        stop_steps = _is_max_limit_reached(self.global_step, self.max_steps)
+        stop_epochs = _is_max_limit_reached(self.current_epoch, self.max_epochs)
 
         should_stop = False
         if self.trainer.should_stop:
@@ -148,35 +173,45 @@ class FitLoop(Loop):
                 )
         self.trainer.should_stop = should_stop
 
-        return stop_steps or should_stop or stop_epochs
+        return stop_steps or should_stop or stop_epochs or self.trainer.num_training_batches == 0
 
     @property
     def skip(self) -> bool:
         """Whether we should skip the training and immediately return from the call to :meth:`run`."""
-        return self.done or self.trainer.num_training_batches == 0
+        # since `trainer.num_training_batches` depends on the `train_dataloader` but that won't be called
+        # until `on_run_start`, we use `limit_train_batches` instead
+        return self.done or self.trainer.limit_train_batches == 0
 
     def connect(self, epoch_loop: TrainingEpochLoop):
         """Connects a training epoch loop to this fit loop."""
         self.epoch_loop = epoch_loop
 
     def reset(self) -> None:
-        """Resets the internal state of this loop"""
+        """Resets the internal state of this loop."""
+        if self.restarting:
+            self.epoch_progress.reset_on_restart()
 
     def on_run_start(self) -> None:
         """Calls the ``on_train_start`` hook."""
+        # reset train dataloader and val dataloader
+        self.trainer.reset_train_val_dataloaders(self.trainer.lightning_module)
+        self._is_fresh_start_epoch = True
         self._results.to(device=self.trainer.lightning_module.device)
         self.trainer.call_hook("on_train_start")
 
     def on_advance_start(self) -> None:
-        """Prepares the dataloader for training and calls the hooks ``on_epoch_start`` and ``on_train_epoch_start``"""
+        """Prepares the dataloader for training and calls the hooks ``on_epoch_start`` and
+        ``on_train_epoch_start``"""
         model = self.trainer.lightning_module
 
         # reset train dataloader
-        if self.current_epoch != 0 and self.trainer._should_reload_dl_epoch:
+        if not self._is_fresh_start_epoch and self.trainer._should_reload_dl_epoch:
             self.trainer.reset_train_dataloader(model)
+        self._is_fresh_start_epoch = False
 
-        # TODO: specify the possible exception
-        with suppress(Exception):
+        if self.trainer.train_dataloader is not None and callable(
+            getattr(self.trainer.train_dataloader.sampler, "set_epoch", None)
+        ):
             # set seed for distributed sampler (enables shuffling for each epoch)
             self.trainer.train_dataloader.sampler.set_epoch(self.current_epoch)
 
@@ -192,15 +227,11 @@ class FitLoop(Loop):
 
     def advance(self) -> None:
         """Runs one whole epoch."""
-        train_dataloader = self.trainer.accelerator.process_dataloader(self.trainer.train_dataloader)
-        train_dataloader = self.trainer.data_connector.get_profiled_train_dataloader(train_dataloader)
+        dataloader = self.trainer.training_type_plugin.process_dataloader(self.trainer.train_dataloader)
+        data_fetcher = self.trainer._data_connector.get_profiled_dataloader(dataloader)
 
         with self.trainer.profiler.profile("run_training_epoch"):
-            # run train epoch
-            epoch_output = self.epoch_loop.run(train_dataloader)
-
-            if epoch_output is None:
-                return
+            self.epoch_loop.run(data_fetcher)
 
             # the global step is manually decreased here due to backwards compatibility with existing loggers
             # as they expect that the same step is used when logging epoch end metrics even when the batch loop has
@@ -215,31 +246,22 @@ class FitLoop(Loop):
         self.epoch_progress.increment_completed()
 
     def on_run_end(self) -> None:
-        """Calls the ``on_train_end`` hook"""
-        # NOTE: the iteration_count/current_epoch is already incremented
+        """Calls the ``on_train_end`` hook."""
+        # NOTE: the current_epoch is already incremented
         # Lightning today does not increment the current epoch at the last epoch run in Trainer.fit
         # To simulate that current behavior, we decrement here.
         # TODO: must be fixed by https://github.com/PyTorchLightning/pytorch-lightning/issues/5007
-        self.current_epoch -= 1
+        self.current_epoch = max(self.current_epoch - 1, 0)
 
         # hook
         self.trainer.call_hook("on_train_end")
 
-        # todo: TPU 8 cores hangs in flush with TensorBoard. Might do for all loggers.
-        # It might be related to xla tensors blocked when moving the cpu
-        # kill loggers
-        if self.trainer.logger is not None:
-            self.trainer.logger.finalize("success")
-
-        # summarize profile results
-        self.trainer.profiler.describe()
-
         # give accelerators a chance to finish
-        self.trainer.accelerator.on_train_end()
-
-    def should_accumulate(self) -> bool:
-        """Whether the gradients should be accumulated"""
-        return self.epoch_loop.batch_loop.should_accumulate()
+        self.trainer.training_type_plugin.on_train_end()
 
     def teardown(self) -> None:
         self.epoch_loop.teardown()
+
+    def _should_accumulate(self) -> bool:
+        """Whether the gradients should be accumulated."""
+        return self.epoch_loop._should_accumulate()
