@@ -45,8 +45,8 @@ _PROFILER = Union[torch.autograd.profiler.profile, torch.cuda.profiler.profile,
 
 
 class RegisterRecordFunction:
-    """
-    While profiling autograd operations, this class will add labels for module names around the forward function.
+    """While profiling autograd operations, this class will add labels for module names around the forward
+    function.
 
     The Lightning PyTorch Profiler will activate this feature automatically. It can be deactivated as follows:
 
@@ -100,10 +100,8 @@ class RegisterRecordFunction:
 
 
 class ScheduleWrapper:
-    """
-    This class is used to override the schedule logic from the profiler and perform
-    recording for both `training_step`, `validation_step`.
-    """
+    """This class is used to override the schedule logic from the profiler and perform recording for both
+    `training_step`, `validation_step`."""
 
     def __init__(self, schedule: Callable) -> None:
         if not _KINETO_AVAILABLE:
@@ -119,11 +117,11 @@ class ScheduleWrapper:
 
     def reset(self):
         # handle properly `fast_dev_run`. PyTorch Profiler will fail otherwise.
-        self._num_optimizer_step_and_closure = 0
+        self._num_optimizer_step_with_closure = 0
         self._num_validation_step = 0
         self._num_test_step = 0
         self._num_predict_step = 0
-        self._optimizer_step_and_closure_reached_end = False
+        self._optimizer_step_with_closure_reached_end = False
         self._validation_step_reached_end = False
         self._test_step_reached_end = False
         self._predict_step_reached_end = False
@@ -134,13 +132,13 @@ class ScheduleWrapper:
     @property
     def is_training(self) -> bool:
         return self._current_action is not None and (
-            self._current_action.startswith("optimizer_step_and_closure_") or self._current_action == "training_step"
+            self._current_action.startswith("optimizer_step_with_closure_") or self._current_action == "training_step"
         )
 
     @property
     def num_step(self) -> int:
         if self.is_training:
-            return self._num_optimizer_step_and_closure
+            return self._num_optimizer_step_with_closure
         if self._current_action == "validation_step":
             return self._num_validation_step
         if self._current_action == "test_step":
@@ -151,10 +149,10 @@ class ScheduleWrapper:
 
     def _step(self) -> None:
         if self.is_training:
-            self._num_optimizer_step_and_closure += 1
+            self._num_optimizer_step_with_closure += 1
         elif self._current_action == "validation_step":
             if self._start_action_name == "on_fit_start":
-                if self._num_optimizer_step_and_closure > 0:
+                if self._num_optimizer_step_with_closure > 0:
                     self._num_validation_step += 1
             else:
                 self._num_validation_step += 1
@@ -166,7 +164,7 @@ class ScheduleWrapper:
     @property
     def has_finished(self) -> bool:
         if self.is_training:
-            return self._optimizer_step_and_closure_reached_end
+            return self._optimizer_step_with_closure_reached_end
         if self._current_action == "validation_step":
             return self._validation_step_reached_end
         if self._current_action == "test_step":
@@ -184,7 +182,7 @@ class ScheduleWrapper:
         action = self._schedule(max(self.num_step, 0))
         if action == ProfilerAction.RECORD_AND_SAVE:
             if self.is_training:
-                self._optimizer_step_and_closure_reached_end = True
+                self._optimizer_step_with_closure_reached_end = True
             elif self._current_action == "validation_step":
                 self._validation_step_reached_end = True
             elif self._current_action == "test_step":
@@ -204,9 +202,9 @@ class PyTorchProfiler(BaseProfiler):
         "test_step",
         "predict_step",
     }
-    RECORD_FUNCTION_PREFIX = "optimizer_step_and_closure_"
+    RECORD_FUNCTION_PREFIX = "optimizer_step_with_closure_"
     STEP_FUNCTIONS = {"training_step", "validation_step", "test_step", "predict_step"}
-    STEP_FUNCTION_PREFIX = "optimizer_step_and_closure_"
+    STEP_FUNCTION_PREFIX = "optimizer_step_with_closure_"
     AVAILABLE_SORT_KEYS = {
         "cpu_time",
         "cuda_time",
@@ -231,12 +229,10 @@ class PyTorchProfiler(BaseProfiler):
         sort_by_key: Optional[str] = None,
         record_functions: Set[str] = None,
         record_module_names: bool = True,
-        profiled_functions: Optional[List] = None,
-        output_filename: Optional[str] = None,
         **profiler_kwargs: Any,
     ) -> None:
-        """
-        This profiler uses PyTorch's Autograd Profiler and lets you inspect the cost of
+        """This profiler uses PyTorch's Autograd Profiler and lets you inspect the cost of.
+
         different operators inside your model - both on the CPU and GPU
 
         Args:
@@ -284,18 +280,19 @@ class PyTorchProfiler(BaseProfiler):
                 If arg ``schedule`` is not a ``Callable``.
                 If arg ``schedule`` does not return a ``torch.profiler.ProfilerAction``.
         """
-        super().__init__(dirpath=dirpath, filename=filename, output_filename=output_filename)
+        super().__init__(dirpath=dirpath, filename=filename)
 
         self._group_by_input_shapes = group_by_input_shapes and profiler_kwargs.get("record_shapes", False)
         self._emit_nvtx = emit_nvtx
         self._export_to_chrome = export_to_chrome
         self._row_limit = row_limit
         self._sort_by_key = sort_by_key or f"{'cuda' if profiler_kwargs.get('use_cuda', False) else 'cpu'}_time_total"
-        self._user_record_functions = set(record_functions or set())
+        self._user_record_functions = record_functions or set()
         self._record_functions_start = self._user_record_functions | self.START_RECORD_FUNCTIONS
         self._record_functions = self._user_record_functions | self.RECORD_FUNCTIONS
         self._record_module_names = record_module_names
         self._profiler_kwargs = profiler_kwargs
+
         self.profiler: Optional[_PROFILER] = None
         self.function_events: Optional["EventList"] = None
         self._lightning_module: Optional["LightningModule"] = None  # set by ProfilerConnector
@@ -366,7 +363,7 @@ class PyTorchProfiler(BaseProfiler):
             # close profiler if it is already opened. might happen if 2 profilers
             # are created and the first one did not call `describe`
             try:
-                torch.autograd._disable_profiler()  # noqa
+                torch.autograd._disable_profiler()
             except (AttributeError, RuntimeError):
                 pass
 
@@ -386,8 +383,8 @@ class PyTorchProfiler(BaseProfiler):
                 self._register.__enter__()
 
         if self._lightning_module is not None:
-            # when the model is used in automatic optimization,
-            # we use `optimizer_step_and_closure` to step the model.
+            # when the model is used in automatic optimization, we use `optimizer_step_with_closure` to step the model.
+            # this profiler event is generated in the `LightningOptimizer.step` method
             if self._lightning_module.automatic_optimization and "training_step" in self.STEP_FUNCTIONS:
                 self.STEP_FUNCTIONS.remove("training_step")
 
